==> mule-devkit-plugin-expression-language-3.4.4-sources.jar: org/mule/devkit/generation/expressionlanguage/ExpressionLanguageAnnotationVerifier.java <==
/**
 * Mule Development Kit
 * Copyright 2010-2012 (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * This software is protected under international copyright law. All use of this software is
 * subject to MuleSoft's Master Subscription Agreement (or other master license agreement)
 * separately entered into in writing between you and MuleSoft. If such an agreement is not
 * in place, you may not use the software.
 */


package org.mule.devkit.generation.expressionlanguage;

import org.mule.api.annotations.ExpressionEnricher;
import org.mule.api.annotations.ExpressionEvaluator;
import org.mule.api.annotations.param.CorrelationId;
import org.mule.api.annotations.param.CorrelationSequence;
import org.mule.api.annotations.param.ExceptionPayload;
import org.mule.api.annotations.param.InboundHeaders;
import org.mule.api.annotations.param.InvocationHeaders;
import org.mule.api.annotations.param.OutboundHeaders;
import org.mule.api.annotations.param.Payload;
import org.mule.api.annotations.param.SessionHeaders;
import org.mule.devkit.generation.api.AnnotationVerificationException;
import org.mule.devkit.generation.api.ModuleAnnotationVerifier;
import org.mule.devkit.model.Method;
import org.mule.devkit.model.Parameter;
import org.mule.devkit.model.Type;
import org.mule.devkit.model.module.Module;
import org.mule.devkit.model.module.ModuleKind;

public class ExpressionLanguageAnnotationVerifier implements ModuleAnnotationVerifier {
    @Override
    public boolean shouldVerify(Module module) {
        return module.getKind() == ModuleKind.EXPRESSION_LANGUAGE;
    }

    @Override
    public void verify(Module module) throws AnnotationVerificationException {
        if (module.getMethodsAnnotatedWith(ExpressionEvaluator.class).size() > 1) {
            throw new AnnotationVerificationException(module, "An @ExpressionLanguage can only contain one @ExpressionEvaluator.");
        }
        if (module.getMethodsAnnotatedWith(ExpressionEnricher.class).size() > 1) {
            throw new AnnotationVerificationException(module, "An @ExpressionLanguage can only contain one @ExpressionEnricher.");
        }
        if (module.getMethodsAnnotatedWith(ExpressionEvaluator.class).size() == 0 &&
                module.getMethodsAnnotatedWith(ExpressionEnricher.class).size() == 0) {
            throw new AnnotationVerificationException(module, "An @ExpressionLanguage must contain one @ExpressionEnricher or one @ExpressionEvaluator or both.");
        }
        for (Method<? extends Type> executableElement : module.getMethodsAnnotatedWith(ExpressionEvaluator.class)) {
            if (executableElement.getParameters().size() == 0) {
                throw new AnnotationVerificationException(executableElement, "An @ExpressionEvaluator must receive at least a String that represents the expression to evaluate.");
            }

            if (executableElement.getReturnType().toString().equals("void")) {
                throw new AnnotationVerificationException(executableElement, "@ExpressionEvaluator cannot be void");
            }

            boolean expressionStringFound = false;
            for (Parameter parameter : executableElement.getParameters()) {
                if (parameter.getAnnotation(Payload.class) == null &&
                        parameter.getAnnotation(OutboundHeaders.class) == null &&
                        parameter.getAnnotation(InboundHeaders.class) == null &&
                        parameter.getAnnotation(SessionHeaders.class) == null &&
                        parameter.getAnnotation(InvocationHeaders.class) == null &&
                        parameter.getAnnotation(ExceptionPayload.class) == null &&
                        parameter.getAnnotation(CorrelationId.class) == null &&
                        parameter.getAnnotation(CorrelationSequence.class) == null) {
                    if (parameter.asTypeMirror().toString().contains("String")) {
                        if (expressionStringFound) {
                            throw new AnnotationVerificationException(executableElement, "An @ExpressionEvaluator can receive only one String and the rest of the arguments must be annotated with either @Payload, @InboundHeaders, @OutboundHeaders, @SessionHeader or @InvocationHeaders.");
                        }
                        expressionStringFound = true;
                    } else {
                        throw new AnnotationVerificationException(executableElement, "An @ExpressionEvaluator can receive only one String and the rest of the arguments must be annotated with either @Payload, @InboundHeaders, @OutboundHeaders, @SessionHeader or @InvocationHeaders.");
                    }
                }
            }
        }

        for (Method<? extends Type> executableElement : module.getMethodsAnnotatedWith(ExpressionEnricher.class)) {
            if (executableElement.getParameters().size() == 0) {
                throw new AnnotationVerificationException(executableElement, "An @ExpressionEnricher must receive at least a String that represents the expression and Object that represents the object to be used for enrichment.");
            }

            if (!executableElement.getReturnType().toString().equals("void")) {
                throw new AnnotationVerificationException(executableElement, "@ExpressionEnricher must be void");
            }

            boolean expressionStringFound = false;
            boolean enrichObjectFound = false;
            for (Parameter parameter : executableElement.getParameters()) {
                if (parameter.getAnnotation(Payload.class) == null &&
                        parameter.getAnnotation(OutboundHeaders.class) == null &&
                        parameter.getAnnotation(InboundHeaders.class) == null &&
                        parameter.getAnnotation(SessionHeaders.class) == null &&
                        parameter.getAnnotation(InvocationHeaders.class) == null &&
                        parameter.getAnnotation(ExceptionPayload.class) == null &&
                        parameter.getAnnotation(CorrelationId.class) == null &&
                        parameter.getAnnotation(CorrelationSequence.class) == null) {
                    if (parameter.asTypeMirror().toString().contains("String")) {
                        if (expressionStringFound) {
                            throw new AnnotationVerificationException(executableElement, "An @ExpressionEnricher can receive only one Object and one String and the rest of the arguments must be annotated with either @Payload, @InboundHeaders, @OutboundHeaders, @SessionHeader or @InvocationHeaders.");
                        }
                        expressionStringFound = true;
                    } else if (parameter.asTypeMirror().toString().contains("Object")) {
                        if (enrichObjectFound) {
                            throw new AnnotationVerificationException(executableElement, "An @ExpressionEnricher can receive only one Object and one String and the rest of the arguments must be annotated with either @Payload, @InboundHeaders, @OutboundHeaders, @SessionHeader or @InvocationHeaders.");
                        }
                        enrichObjectFound = true;
                    } else {
                        throw new AnnotationVerificationException(executableElement, "An @ExpressionEnricher can receive only one Object and one String and the rest of the arguments must be annotated with either @Payload, @InboundHeaders, @OutboundHeaders, @SessionHeader or @InvocationHeaders.");
                    }
                }
            }

        }
    }
}

==> mule-devkit-plugin-expression-language-3.5.1-sources.jar: org/mule/devkit/generation/expressionlanguage/ExpressionLanguageAnnotationVerifier.java <==
/**
 * Mule Development Kit
 * Copyright 2010-2012 (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 *
 * This software is protected under international copyright law. All use of this software is
 * subject to MuleSoft's Master Subscription Agreement (or other master license agreement)
 * separately entered into in writing between you and MuleSoft. If such an agreement is not
 * in place, you may not use the software.
 */


package org.mule.devkit.generation.expressionlanguage;

import org.mule.api.annotations.ExpressionEnricher;
import org.mule.api.annotations.ExpressionEvaluator;
import org.mule.api.annotations.param.CorrelationId;
import org.mule.api.annotations.param.CorrelationSequence;
import org.mule.api.annotations.param.ExceptionPayload;
import org.mule.api.annotations.param.InboundHeaders;
import org.mule.api.annotations.param.InvocationHeaders;
import org.mule.api.annotations.param.OutboundHeaders;
import org.mule.api.annotations.param.Payload;
import org.mule.api.annotations.param.SessionHeaders;
import org.mule.devkit.generation.api.AnnotationVerificationException;
import org.mule.devkit.generation.api.ModuleAnnotationVerifier;
import org.mule.devkit.generation.api.gatherer.*;
import org.mule.devkit.generation.api.gatherer.Message;
import org.mule.devkit.model.Method;
import org.mule.devkit.model.Parameter;
import org.mule.devkit.model.Type;
import org.mule.devkit.model.module.Module;
import org.mule.devkit.model.module.ModuleKind;

public class ExpressionLanguageAnnotationVerifier implements ModuleAnnotationVerifier {
    @Override
    public boolean shouldVerify(Module module) {
        return module.getKind() == ModuleKind.EXPRESSION_LANGUAGE;
    }

    @Override
    public void verify(Module module,NotificationGatherer gatherer) throws AnnotationVerificationException {
        if (module.getMethodsAnnotatedWith(ExpressionEvaluator.class).size() > 1) {
            gatherer.error(module, Message.DEVKIT_134);
        }
        if (module.getMethodsAnnotatedWith(ExpressionEnricher.class).size() > 1) {
            gatherer.error(module, Message.DEVKIT_135);
        }
        if (module.getMethodsAnnotatedWith(ExpressionEvaluator.class).size() == 0 &&
                module.getMethodsAnnotatedWith(ExpressionEnricher.class).size() == 0) {
            gatherer.error(module, Message.DEVKIT_136);
        }
        for (Method<? extends Type> executableElement : module.getMethodsAnnotatedWith(ExpressionEvaluator.class)) {
            if (executableElement.getParameters().size() == 0) {
                gatherer.error(executableElement, Message.DEVKIT_137);
            }

            if (executableElement.getReturnType().toString().equals("void")) {
                gatherer.error(executableElement, Message.DEVKIT_138);
            }

            boolean expressionStringFound = false;
            for (Parameter parameter : executableElement.getParameters()) {
                if (!parameter.hasAnnotation(Payload.class) &&
                        !parameter.hasAnnotation(OutboundHeaders.class) &&
                        !parameter.hasAnnotation(InboundHeaders.class) &&
                        !parameter.hasAnnotation(SessionHeaders.class) &&
                        !parameter.hasAnnotation(InvocationHeaders.class) &&
                        !parameter.hasAnnotation(ExceptionPayload.class) &&
                        !parameter.hasAnnotation(CorrelationId.class) &&
                        !parameter.hasAnnotation(CorrelationSequence.class) ) {
                    if (parameter.asTypeMirror().toString().contains("String")) {
                        if (expressionStringFound) {
                            gatherer.error(executableElement, Message.DEVKIT_139);
                            continue;
                        }
                        expressionStringFound = true;
                    } else {
                        gatherer.error(executableElement, Message.DEVKIT_140);
                    }
                }
            }
        }

        for (Method<? extends Type> executableElement : module.getMethodsAnnotatedWith(ExpressionEnricher.class)) {
            if (executableElement.getParameters().size() == 0) {
                gatherer.error(executableElement, Message.DEVKIT_141);
            }

            if (!executableElement.getReturnType().toString().equals("void")) {
                gatherer.error(executableElement, Message.DEVKIT_142);
            }

            boolean expressionStringFound = false;
            boolean enrichObjectFound = false;
            for (Parameter parameter : executableElement.getParameters()) {
                if (!parameter.hasAnnotation(Payload.class) &&
                        !parameter.hasAnnotation(OutboundHeaders.class) &&
                        !parameter.hasAnnotation(InboundHeaders.class) &&
                        !parameter.hasAnnotation(SessionHeaders.class) &&
                        !parameter.hasAnnotation(InvocationHeaders.class) &&
                        !parameter.hasAnnotation(ExceptionPayload.class) &&
                        !parameter.hasAnnotation(CorrelationId.class) &&
                        !parameter.hasAnnotation(CorrelationSequence.class) ) {
                    if (parameter.asTypeMirror().toString().contains("String")) {
                        if (expressionStringFound) {
                            gatherer.error(executableElement, Message.DEVKIT_143);
                            continue;
                        }
                        expressionStringFound = true;
                    } else if (parameter.asTypeMirror().toString().contains("Object")) {
                        if (enrichObjectFound) {
                            gatherer.error(executableElement, Message.DEVKIT_144);
                            continue;
                        }
                        enrichObjectFound = true;
                    } else {
                        gatherer.error(executableElement, Message.DEVKIT_145);
                    }
                }
            }

        }
    }
}
